@@ -1,9 +1,11 @@
+//! Implementation of AMQP1.0 receiver
+
 use std::{marker::PhantomData, sync::Arc};
 
 use bytes::BytesMut;
 use fe2o3_amqp_types::{
     definitions::{self, AmqpError, DeliveryNumber, DeliveryTag, SequenceNo},
-    messaging::{Accepted, Address, DeliveryState, Modified, Rejected, Released, Target},
+    messaging::{Accepted, Address, DeliveryState, Modified, Rejected, Released},
     performatives::Transfer,
 };
 use futures_util::StreamExt;
@@ -14,7 +16,7 @@ use tokio_util::sync::PollSender;
 use crate::{
     control::SessionControl,
     endpoint::Link,
-    link::error::{detach_error_expecting_frame, map_send_detach_error},
+    link::error::detach_error_expecting_frame,
     session::{self, SessionHandle},
     Payload,
 };
@@ -22,7 +24,7 @@ use crate::{
 use super::{
     builder::{self, WithTarget, WithoutName},
     delivery::Delivery,
-    error::DetachError,
+    error::{AttachError, DetachError},
     receiver_link::section_number_and_offset,
     role,
     state::LinkFlowState,
@@ -36,10 +38,14 @@ macro_rules! or_assign {
             Some(value) => {
                 if let Some(other_value) = $other.$field {
                     if *value != other_value {
-                        return Err(Error::AmqpError {
-                            condition: AmqpError::NotAllowed,
-                            description: Some(format!("Inconsistent {:?} in multi-frame delivery", value))
-                        })
+                        return Err(Error::Local(
+                            definitions::Error::new(
+                                AmqpError::NotAllowed,
+                                Some(format!("Inconsistent {:?} in multi-frame delivery", value)),
+                                None
+                            )
+                        ))
+
                     }
                 }
             },
@@ -141,9 +147,14 @@ impl IncompleteTransfer {
 type ReceiverFlowState = LinkFlowState<role::Receiver>;
 type ReceiverLink = super::Link<role::Receiver, Arc<ReceiverFlowState>, DeliveryState>;
 
+/// Credit mode for the link
 #[derive(Debug, Clone)]
 pub enum CreditMode {
+    /// Manual mode will require the user to manually allocate credit whenever
+    /// the available credits are depleted
     Manual,
+
+    /// The receiver will automatically re-fill the credit
     Auto(SequenceNo),
 }
 
@@ -154,6 +165,45 @@ impl Default for CreditMode {
     }
 }
 
+/// An AMQP1.0 receiver
+///
+/// # Attach a new receiver with default configurations
+/// 
+/// ```rust, ignore
+/// let mut receiver = Receiver::attach(
+///     &mut session,           // mutable reference to SessionHandle
+///     "rust-receiver-link-1", // link name
+///     "q1"                    // Source address
+/// ).await.unwrap();
+/// ```
+/// 
+/// ## Default configuration
+/// 
+/// | Field | Default Value |
+/// |-------|---------------|
+/// |`name`|`String::default()`|
+/// |`snd_settle_mode`|`SenderSettleMode::Mixed`|
+/// |`rcv_settle_mode`|`ReceiverSettleMode::First`|
+/// |`source`|`None` |
+/// |`target`| `Some(Target)` |
+/// |`initial_delivery_count`| `0` |
+/// |`max_message_size`| `None` |
+/// |`offered_capabilities`| `None` |
+/// |`desired_capabilities`| `None` |
+/// |`Properties`| `None` |
+/// |`buffer_size`| `u16::MAX` |
+/// |`role`| `role::Sender` |
+///
+/// # Customize configuration with [`builder::Builder`]
+/// 
+/// ```rust, ignore
+/// let mut receiver = Receiver::builder()
+///     .name("rust-receiver-link-1")
+///     .source("q1")
+///     .attach(&mut session)
+///     .await
+///     .unwrap();
+/// ```
 #[derive(Debug)]
 pub struct Receiver<S> {
     pub(crate) link: ReceiverLink,
@@ -175,15 +225,45 @@ pub struct Receiver<S> {
 }
 
 impl Receiver<Detached> {
+    /// Creates a builder for the [`Receiver`]
     pub fn builder() -> builder::Builder<role::Receiver, WithoutName, WithTarget> {
-        builder::Builder::new().target(Target::builder().build())
+        builder::Builder::<role::Receiver, _, _>::new()
     }
 
+    /// Attach the receiver link to a session with the default configuration
+    ///
+    /// # Default configuration
+    /// 
+    /// | Field | Default Value |
+    /// |-------|---------------|
+    /// |`name`|`String::default()`|
+    /// |`snd_settle_mode`|`SenderSettleMode::Mixed`|
+    /// |`rcv_settle_mode`|`ReceiverSettleMode::First`|
+    /// |`source`|`None` |
+    /// |`target`| `Some(Target)` |
+    /// |`initial_delivery_count`| `0` |
+    /// |`max_message_size`| `None` |
+    /// |`offered_capabilities`| `None` |
+    /// |`desired_capabilities`| `None` |
+    /// |`Properties`| `None` |
+    /// |`buffer_size`| `u16::MAX` |
+    /// |`role`| `role::Sender` |
+    /// 
+    ///  
+    /// # Example
+    /// 
+    /// ```rust, ignore
+    /// let mut receiver = Receiver::attach(
+    ///     &mut session,           // mutable reference to SessionHandle
+    ///     "rust-receiver-link-1", // link name
+    ///     "q1"                    // Source address
+    /// ).await.unwrap();
+    /// ```
     pub async fn attach(
         session: &mut SessionHandle,
         name: impl Into<String>,
         addr: impl Into<Address>,
-    ) -> Result<Receiver<Attached>, Error> {
+    ) -> Result<Receiver<Attached>, AttachError> {
         Self::builder()
             .name(name)
             .source(addr)
@@ -229,7 +309,10 @@ impl Receiver<Detached> {
         if let Err(err) =
             super::do_attach(&mut self.link, &mut self.outgoing, &mut self.incoming).await
         {
-            return Err(map_send_detach_error(err, self));
+            return Err(match DetachError::try_from((self, err.into())) {
+                Ok(err) => err,
+                Err(_) => unreachable!(),
+            });
         }
 
         Ok(Receiver::<Attached> {
@@ -263,6 +346,14 @@ impl Receiver<Attached> {
         }
     }
 
+    /// Receive a message from the link
+    ///
+    /// # Example
+    ///
+    /// ```rust, ignore
+    /// let delivery: Delivery<String> = receiver.recv::<String>().await.unwrap();
+    /// receiver.accept(&delivery).await.unwrap();
+    /// ```
     pub async fn recv<T>(&mut self) -> Result<Delivery<T>, Error>
     where
         T: for<'de> serde::Deserialize<'de> + Send,
@@ -280,9 +371,12 @@ impl Receiver<Attached> {
     where
         T: for<'de> serde::Deserialize<'de> + Send,
     {
-        let frame = self.incoming.next().await.ok_or_else(|| Error::AmqpError {
-            condition: AmqpError::IllegalState,
-            description: Some("Session is dropped".into()),
+        let frame = self.incoming.next().await.ok_or_else(|| {
+            Error::Local(definitions::Error::new(
+                AmqpError::IllegalState,
+                Some("Session is dropped".into()),
+                None,
+            ))
         })?;
 
         match frame {
@@ -299,10 +393,11 @@ impl Receiver<Attached> {
                 payload,
             } => self.on_incoming_transfer(performative, payload).await,
             LinkFrame::Attach(_) => {
-                return Err(Error::AmqpError {
-                    condition: AmqpError::IllegalState,
-                    description: Some("Received Attach on an attached link".into()),
-                })
+                return Err(Error::Local(definitions::Error::new(
+                    AmqpError::IllegalState,
+                    Some("Received Attach on an attached link".into()),
+                    None,
+                )))
             }
             LinkFrame::Flow(_) | LinkFrame::Disposition(_) => {
                 // Flow and Disposition are handled by LinkHandle which runs
@@ -419,7 +514,10 @@ impl Receiver<Attached> {
             .await
     }
 
-    // TODO: how to stop draining? `set_credit` will stop draining
+    /// Drain the link.
+    ///
+    /// This will send a `Flow` performative with the `drain` field set to true.
+    /// Setting the credit will set the `drain` field to false and stop draining
     pub async fn drain(&mut self) -> Result<(), Error> {
         use crate::endpoint::ReceiverLink;
 
@@ -436,7 +534,11 @@ impl Receiver<Attached> {
             .await
     }
 
-    // TODO: reduce mostly duplicated code (Sender/Receiver::detach/close)?
+    /// Detach the link.
+    ///
+    /// This will send a `Detach` performative with the `closed` field set to false. If the remote
+    /// peer responds with a Detach performative whose `closed` field is set to true, the link will
+    /// re-attach and then close by exchanging closing Detach performatives.
     pub async fn detach(self) -> Result<Receiver<Detached>, DetachError<Receiver<Detached>>> {
         let mut detaching = Receiver::<Detached> {
             link: self.link,
@@ -452,14 +554,16 @@ impl Receiver<Attached> {
         };
 
         // Send a non-closing detach
-        match detaching
+        if let Err(err) = detaching
             .link
             .send_detach(&mut detaching.outgoing, false, None)
             .await
         {
-            Ok(_) => {}
-            Err(e) => return Err(map_send_detach_error(e, detaching)),
-        };
+            match DetachError::try_from((detaching, err)) {
+                Ok(error) => return Err(error),
+                Err(_) => unreachable!(),
+            }
+        }
 
         // Wait for remote detach
         let frame = match detaching.incoming.next().await {
@@ -510,10 +614,18 @@ impl Receiver<Attached> {
             .await
         {
             Ok(_) => Ok(detaching),
-            Err(e) => return Err(map_send_detach_error(e, detaching)),
+            Err(e) => {
+                return Err(match DetachError::try_from((detaching, e.into())) {
+                    Ok(err) => err,
+                    Err(_) => unreachable!(),
+                })
+            }
         }
     }
 
+    /// Close the link.
+    ///
+    /// This will send a Detach performative with the `closed` field set to true.
     pub async fn close(self) -> Result<(), DetachError<Receiver<Detached>>> {
         let mut detaching = self.into_detached();
         let link_name = detaching.link.name.clone();
@@ -526,7 +638,12 @@ impl Receiver<Attached> {
             .await
         {
             Ok(_) => {}
-            Err(e) => return Err(map_send_detach_error(e, detaching)),
+            Err(e) => {
+                return Err(match DetachError::try_from((detaching, e.into())) {
+                    Ok(err) => err,
+                    Err(_) => unreachable!(),
+                })
+            }
         }
 
         // Wait for remote detach
@@ -583,7 +700,12 @@ impl Receiver<Attached> {
                 .await
             {
                 Ok(_) => detaching,
-                Err(err) => return Err(map_send_detach_error(err, detaching)),
+                Err(e) => {
+                    return Err(match DetachError::try_from((detaching, e.into())) {
+                        Ok(err) => err,
+                        Err(_) => unreachable!(),
+                    })
+                }
             }
         };
 
@@ -594,7 +716,12 @@ impl Receiver<Attached> {
             .await
         {
             Ok(_) => {}
-            Err(e) => return Err(map_send_detach_error(e, detaching)),
+            Err(e) => {
+                return Err(match DetachError::try_from((detaching, e.into())) {
+                    Ok(err) => err,
+                    Err(_) => unreachable!(),
+                })
+            }
         }
 
         Ok(())
@@ -611,9 +738,9 @@ impl Receiver<Attached> {
 //     }
 // }
 
-/// TODO: Use type state to differentiate Mode First and Mode Second?
+// TODO: Use type state to differentiate Mode First and Mode Second?
 impl Receiver<Attached> {
-    /// TODO: batch disposition
+    // TODO: batch disposition
     async fn dispose(
         &mut self,
         delivery_id: DeliveryNumber,
@@ -637,6 +764,8 @@ impl Receiver<Attached> {
         Ok(())
     }
 
+    /// Accept the message by sending a disposition with the `delivery_state` field set
+    /// to `Accept`
     pub async fn accept<T>(&mut self, delivery: &Delivery<T>) -> Result<(), Error> {
         let state = DeliveryState::Accepted(Accepted {});
         self.dispose(
@@ -647,6 +776,8 @@ impl Receiver<Attached> {
         .await
     }
 
+    /// Reject the message by sending a disposition with the `delivery_state` field set
+    /// to `Reject`
     pub async fn reject<T>(
         &mut self,
         delivery: &Delivery<T>,
@@ -663,6 +794,8 @@ impl Receiver<Attached> {
         .await
     }
 
+    /// Release the message by sending a disposition with the `delivery_state` field set
+    /// to `Release`
     pub async fn release<T>(&mut self, delivery: &Delivery<T>) -> Result<(), Error> {
         let state = DeliveryState::Released(Released {});
         self.dispose(
@@ -673,6 +806,8 @@ impl Receiver<Attached> {
         .await
     }
 
+    /// Modify the message by sending a disposition with the `delivery_state` field set
+    /// to `Modify`
     pub async fn modify<T>(
         &mut self,
         delivery: &Delivery<T>,
